import java.io.*;

/**
 * This class provides a simple wrapper for PrintWriter.
 * It is used to provide indentation for recursively
 * generated programs.
 *
 * @author Craig Cleaveland
 * @version 1.0
 */
public class ProgramWriter extends PrintWriter {
    
    /**
     * Creates a new ProgramWriter from a Writer object.
     */
    public ProgramWriter(Writer w) {
        super(w);
    }
    
    /**
     * Creates a new ProgramWriter from an OutputStream object.
     */
    public ProgramWriter(OutputStream o) {
        super(o);
    }
    
    String indent = "";
    
    /**
     * Sets the indentation.
     * ProgramWriter objects will prefix each line with the
     * Indent string which is typically blank characters.
     * It is used to help format generated programs that are
     * created recursively, so that each recursion indents the
     * generated code.
     */
    public void setIndent(String s) {
        indent = s;
    }
    
    /**
     * Returns the current Indent string.
     */
    public String getIndent() {
        return indent;
    }
    
    public void print(char ch) {
        if (ch=='\n') {
            super.print(ch+indent);
        } else {
            super.print(ch);
        }
    }
    
    /**
     * Prints the string to the output.
     * Each newline character is prefixed with the Indent string.
     */
    public void print(String str) {
        while (true) {
            int j = str.indexOf('\n');
            if (j<0) {
                super.print(str);
                return;
            }
            super.print(str.substring(0, j+1));
            super.print(indent);
            str = str.substring(j+1);
        }
    }
    
    /**
     * Prints the string and a new line character.
     */
    public void println(String s) {
        print(s);
        print('\n');
    }
}
    